/* CSE142, Spring 1999	Dustin Hillard				*
 * Homework 4			Section BG			*
 * Traffic animation		hillard@u.washington.edu	*/


#include "gp142.h"
#include <stdio.h>
#include <math.h>
#include <time.h>
	
/* constants */
#define TRUE    		1
#define FALSE   		0
#define NON_APPLIC 601

/* The constant FILL is used in function  *
 * calls when a solid shape is desired.   */
#define FILL 0

/* characteristics for car procedures */
#define MAXIMUM_SPEED 15
#define TOO_CLOSE 90
#define SAFE_ZONE 75
#define TRIAL_TIME 100
#define CAR_CANT_ENTER 5

/* direction of scroll */

#define SCROLL_OUT 1
#define SCROLL_IN 0

/* process of scrolling */

#define SCROLLIN_OUT 2
#define SCROLLIN_IN 0
#define SCROLL_SPEED 5

/* Menu buttons: */
#define ALL 50					/* non-negative numbers outside car range */
#define BACK 51

/* Top-left coordinates of button palette */
#define MENU_TOP		 (GP142_YMAX - 5)
#define MENU_LEFT		 (-GP142_XMAX + 5)
#define SUBMENU_LEFT (-GP142_XMAX + 80)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	75
#define SUBMENU_HEIGHT 15
#define SUBMENU_WIDTH 40

/* Number of buttons */
#define NUMBER_BUTTONS 	9

/*   Mouse click in some button */
#define BUTTON_CLICK 1
#define SUBMENU_CLICK 2

/*   Mouse click elsewhere */
#define OTHER_CLICK 0

/* y-coordinate of the top of the ground in the landscape */
#define HORZN (- 1 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY1_CENTER ((-GP142_YMAX + HORZN) / 2)
#define HIGHWAY2_CENTER ((-GP142_YMAX - 50 + HORZN) / 3)
#define HIGHWAY_WIDTH  10

/* Maximum number of vehicles that can appear in the animation */
#define MAX_VEHICLES	20

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40
#define WHEEL_RADIUS 5

/* Boat attributes */

#define BOAT_LENGTH 50
#define BOAT_HEIGHT 10
#define CREW_HEAD_RADIUS 2
#define CREW_SPACING 7
#define CREW_MEMBERS 6

/* Rain attributes */

#define MAX_DROPS 900
#define RAIN_WIDTH 1

/*****************************************************************************

		 STRUCT PROTOTYPES 

*****************************************************************************/

typedef struct {	int car_counter;						/* counts cars initialized */
									int number_of_cars_for_menu;/* MAX cars for car select */
									int number_of_cars;					/* total cars user allows	 */
									int submenu_status;					/* TRUE when submenu is out*/
									int menu_selection;					/* holds menu selection    */
									int scroll;									/* rect width during scroll*/
									int scrolling_status;				/* TRUE if scrolling in		 */
									int submenu_selection;			/* holds submenu selection */
									int number_of_buttons;			/* holds number of buttons */
									int instruction_type;				/* value for type of inst. */
									int tick_counter;						/* serves as time limit var*/
									int ignore_click;           /* true to ignore clicks   */
									int error_message;					/* true to display error	 */
									int boat_status;						/* true to display boat	   */
									int boat_center_x;					/* boat x coord						 */
									int boat_center_y;					/* boat y coord						 */
}	submenu_data;

typedef struct {		/* Information about vehicles in the animation:*/

	int car_x[MAX_VEHICLES], car_y[MAX_VEHICLES];	/* position of the cars		 */
	int car_direction[MAX_VEHICLES];		  /* direction: +1=right, -1=left		 */
	int car_color[MAX_VEHICLES];				  /* color of the cars							 */
	int car_speed[MAX_VEHICLES];				  /* speed in pixel per tick of cars */
	int speed_holder[MAX_VEHICLES];				/* hold car speed while stopped		 */
	int highway[MAX_VEHICLES];					  /* highway car is on							 */
	int distance_to_others[MAX_VEHICLES]; /* distance from one car to others */
}	car_data;

typedef struct {		/* Information about raindrops */

	int rain_x[MAX_DROPS], rain_y[MAX_DROPS];/* position of the drop					*/
	int length[MAX_DROPS];									 /* length of the drop						*/
	int raindrops_so_far;										 /* holds number of drop displayed*/
	int increment;											 		 /* allow gradual increase of rain*/
	int raining;											 			 /* TRUE if it is raining					*/
}	rain_data;

/*****************************************************************************

     FUNCTION PROTOTYPES

*****************************************************************************/

/* initialize random number generator 																		*/
void initialize_random_numbers();

/* draw the background, including the sun at coordinates sun_x and sun_y	*/
void draw_landscape(int sun_x, int sun_y, int *sky_color);

/* draw crew boat																												  */
void draw_boat(submenu_data *current);

/* make it rain																														*/
void make_it_rain(rain_data *rain);

/* draw instructions																											*/
void draw_instructions(submenu_data *current, rain_data *rain);

/* draw the buttons 																											*/
void draw_buttons(submenu_data *current);

/* draw submenu buttons																										*/
void draw_submenu(submenu_data *current);
int  scrolling_submenu(int direction, submenu_data *current);

/* return kind of mouse click 																						*/
int  classify_mouse_click(int mouse[2], submenu_data *current);

/* process button click at location x, y,  update car attributes					*/
int handle_button(int mouse[2], car_data *cars, submenu_data *current);

/* process submenu buttons 																								*/
int handle_submenu(int mouse[2], car_data *cars, 
											submenu_data *current, rain_data *rain);

/* update sun coordinates sun_x, sun_y for one tick 											*/
void move_sun(int *sun_x, int *sun_y, int *sky_color);

/* update vehicle positions																								*/
void move_traffic(car_data *cars, submenu_data *current);

/* return distance between 2 cars relative to highway the car is on				*/
int dist_to_next_car(int car1, int car2, 
											car_data *cars, submenu_data *current);

/* slow down car if near the car in front																	*/
int proximity(int car_checked_from, int how_close,
							car_data *cars, submenu_data *current);

/* determine if road is open so a car can come on													*/
int is_road_taken(int car_checked_for, car_data *cars, submenu_data *current);

/* draw all vehicles 																											*/
void draw_traffic(car_data *cars, submenu_data *current);

/* draw a single car 																											*/
void draw_car(car_data *cars, submenu_data *current);

/* display error message if internal error is detected.										*/
void something_is_wrong (void);


/*****************************************************************************

     MAIN

*****************************************************************************/

int main(void)
{
	int  quit;								 /* = "user has selected quit"								*/
	int  event;								 /* the most recent GP142 event								*/
	char key_pressed;					 /* last keyboard character from key event		*/
	int  mouse[2];						 /* array of mouse coordinates								*/
	int  click_kind;					 /* kind of mouse click (button, etc.)				*/	
	submenu_data current;			 /* holds current submenu information					*/
	car_data cars;						 /* holds car characteristics									*/
	rain_data rain;						 /* holds raindrop data												*/

	int sun_x, sun_y;												/* screen x and y coordinates			*/
	int i;																	/* iteration counter							*/
	int sky_color = ICE_BLUE;								/* used by landscape for sky			*/

	initialize_random_numbers();
	
	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.					*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial location of the sun		*/
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 55;

	/* initialize submenu current struct */

	current.car_counter = 0;
	current.number_of_cars = 1;
	current.submenu_status = FALSE;
	current.menu_selection = 0;
	current.scroll = 0;
	current.scrolling_status = FALSE;
	current.tick_counter = 0;
	current.error_message = TRUE;
	current.boat_status = FALSE;
	current.boat_center_x = GP142_XMAX + 35;	/*start center x coord. for boat*/
	current.boat_center_y = -GP142_YMAX + 20; /*start center y coord. for boat*/

	/* initialize cars struct values */

	for(i = 0; i < MAX_VEHICLES; i++){
		cars.speed_holder[i] = 8;
		cars.car_x[i] = 0;
		cars.car_y[i] = 0;
	}

	/* initialize some rain struct values */
	rain.raindrops_so_far = 0;
	rain.increment = 0;
	rain.raining = FALSE;

	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x, sun_y, &sky_color);
	draw_buttons(&current);
	draw_traffic(&cars, &current);

	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse[0], &mouse[1], &key_pressed);
		
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed, not used */
			break;
			
		case GP142_MOUSE:
			/* Mouse click */
			click_kind = classify_mouse_click(mouse, &current);/*obtain click type*/
			if (click_kind == BUTTON_CLICK)	
			 quit = handle_button(mouse, &cars, &current); /*handle main buttons*/
			if (click_kind == SUBMENU_CLICK) 
			 /* handle submenu buttons, scroll in when back selected (status=F) */
			 current.scrolling_status = 
																handle_submenu(mouse, &cars, &current, &rain);
			else { /* do nothing */ }
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event, update things that changes as time advances */
			move_sun(&sun_x, &sun_y, &sky_color);
			move_traffic(&cars, &current);
			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
	/* Redraw everything */
		draw_landscape(sun_x, sun_y, &sky_color);
		draw_buttons(&current);
		if(current.scrolling_status)	/* scroll in when status == TRUE) */
			current.scrolling_status = scrolling_submenu(SCROLL_IN, &current);
		draw_traffic(&cars, &current);
		if(current.boat_status)
			draw_boat(&current);
		if(rain.raining)
			make_it_rain(&rain);
		draw_instructions(&current, &rain);
	}  /* end event loop */
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */


/*****************************************************************************

     BACKGROUND LANDSCAPE

*****************************************************************************/


/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y, int *sky_color)
{
	int i;															/* iteration counter					*/

	/* Draw the sky  */
	GP142_rectangleXY(*sky_color, -GP142_XMAX, -GP142_YMAX,
										GP142_XMAX, GP142_YMAX, FILL);
	
	/* Draw the sun */
	GP142_circleXY(YELLOW, sun_x, sun_y , 20);
	
	/* Draw the ground, highway, and lake */
	GP142_rectangleXY(FOREST_GREEN, -GP142_XMAX, -GP142_YMAX,
										GP142_XMAX, HORZN, FILL);
	GP142_ovalXY(NAVY_BLUE, -GP142_XMAX/2 ,-GP142_YMAX -100, GP142_XMAX*4/3 , 
									 -GP142_YMAX+75, FILL);
	GP142_ovalXY(NAVY_BLUE, -GP142_XMAX*2 ,-GP142_YMAX*2, GP142_XMAX/2 , 
									 -GP142_YMAX/5, FILL);
	GP142_rectangleXY(BLACK, 
									 -GP142_XMAX, HIGHWAY1_CENTER - HIGHWAY_WIDTH,
										GP142_XMAX, HIGHWAY1_CENTER + HIGHWAY_WIDTH,
										FILL);
	GP142_rectangleXY(BLACK, 
									 -GP142_XMAX, HIGHWAY2_CENTER - HIGHWAY_WIDTH,
										GP142_XMAX, HIGHWAY2_CENTER + HIGHWAY_WIDTH,
										FILL);

	/*Draw dotted line in middle of road */
	for(i=-GP142_XMAX; i <=GP142_XMAX; i+=5)
		if(i%2 == 0){
			GP142_lineXY(YELLOW, i, HIGHWAY1_CENTER, 
											i + 4, HIGHWAY1_CENTER, 1);
			GP142_lineXY(YELLOW, i, HIGHWAY2_CENTER, 
											i + 4, HIGHWAY2_CENTER, 1);
		}
	
	
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORZN,
		GP142_XMAX, HORZN,
		0, (4 * GP142_YMAX)/5 ,
		FILL);

	/* snow on grey peak */
	GP142_triangleXY(WHITE, -27, (4 * GP142_YMAX)/5 - 20,
													 27, (4 * GP142_YMAX)/5 - 20,
														0, (4 * GP142_YMAX)/5,
													FILL);
	GP142_triangleXY(WHITE, -27, (4 * GP142_YMAX)/5 - 20,
													-9,  (4 * GP142_YMAX)/5 - 20, 
													-18, (4 * GP142_YMAX)/5 - 25,
													FILL);
	GP142_triangleXY(WHITE, -9,  (4 * GP142_YMAX)/5 - 20,
													 9,  (4 * GP142_YMAX)/5 - 20, 
													 0,  (4 * GP142_YMAX)/5 - 25,
													FILL);
	GP142_triangleXY(WHITE, 9,   (4 * GP142_YMAX)/5 - 20,
													27,  (4 * GP142_YMAX)/5 - 20, 
													18,  (4 * GP142_YMAX)/5 - 25,
													FILL);
	
	/* other mountains */
	GP142_triangleXY(LT_GRAY,
		-( 5 * GP142_XMAX/4), HORZN,
		(GP142_XMAX)/3  , HORZN,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/4 ,
		FILL);
	
	GP142_triangleXY(BROWN,
		0 , HORZN,
		GP142_XMAX*2, HORZN,
		GP142_XMAX-25, (2 * GP142_YMAX)/3 ,
		FILL);
	
} /*end draw_landscape */

/* draw the crew boat */
void draw_boat(submenu_data *current) {

	int boat_front, boat_back;		/* used to hold coords of boat shapes */
	int boat_bottom, boat_top, boat_middle;
	int i;

	boat_front = current->boat_center_x - BOAT_LENGTH/2;
	boat_back = boat_front + BOAT_LENGTH;
	boat_bottom = current->boat_center_y - BOAT_HEIGHT/2;
	boat_top = boat_bottom + BOAT_HEIGHT;
	boat_middle = boat_top - 3;

	/* draw a crew boat */
	GP142_ovalXY(BROWN,
		boat_front, boat_bottom,
		boat_back, boat_top,
		FILL);
	GP142_rectangleXY(NAVY_BLUE,
		boat_front, boat_middle, /* block out top of oval */
		boat_back, boat_top,
		FILL);

	/* draw people */
	for(i=1; i<=CREW_MEMBERS; i++) {

		/* draw head */
		GP142_circleXY(PEACH,
			boat_front + i*CREW_SPACING, boat_top + 3,
			CREW_HEAD_RADIUS);
		/* draw body */
		if(i%2 == 0)
			GP142_ovalXY(GOLD,
			boat_front + i*CREW_SPACING - 3, boat_middle,
			boat_front + i*CREW_SPACING + 3, boat_top,
			FILL);
		else
			GP142_ovalXY(PURPLE,
			boat_front + i*CREW_SPACING - 3, boat_middle,
			boat_front + i*CREW_SPACING + 3, boat_top,
			FILL);
	}


	/* draw oars for boat */
	for(i=1; i<=CREW_MEMBERS; i++) {
		if(abs(current->boat_center_x%3) == 0)
			GP142_lineXY(BROWN,
				boat_front + CREW_SPACING*i, boat_top - BOAT_HEIGHT/2,
				boat_front + CREW_SPACING*i + 8, boat_bottom - 7,
				2);
		if(abs(current->boat_center_x%3) == 1)
			GP142_lineXY(BROWN,
				boat_front + CREW_SPACING*i, boat_top - BOAT_HEIGHT/2,
				boat_front + CREW_SPACING*i + 0, boat_bottom - 7,
				2);
		if(abs(current->boat_center_x%3) == 2)
			GP142_lineXY(BROWN,
				boat_front + CREW_SPACING*i, boat_top - BOAT_HEIGHT/2,
				boat_front + CREW_SPACING*i - 8, boat_bottom - 7,
				2);
	}
	current->boat_center_x -= 2;
	if(current->boat_center_x%5 == 0) /* move up slowly */
		current->boat_center_y += 1;

	if(boat_back < -GP142_XMAX) {
		current->boat_center_x = GP142_XMAX + 35;		/* center x coord. for boat	*/
		current->boat_center_y = -GP142_YMAX + 20;	/* center y coord. for boat	*/
	}
}

/* create rain on the screen */
void make_it_rain(rain_data *rain) {

	int drop;				 /* number of current drop */

	void initialize_random_numbers();
	
	rain->increment += 1;	 /* adds eight drops each time function is called */
	if(rain->increment >= MAX_DROPS) /* up to a maximum set by the constant */
		rain->increment = MAX_DROPS;

	/* initialize up to "increment" of rain */
	for(drop = rain->raindrops_so_far; drop < rain->increment; drop++) {

		void initialize_random_numbers();

		/* intialize drop in quadrant I */
		if((rain->raindrops_so_far%4) == 0) {
			rain->rain_x[drop] = (3*rand())%290 + 1;/*+ one fixes error when == 0*/
			rain->rain_y[drop] = (7*rand())%290 + 1;/*multiply make "more" random*/
			rain->length[drop] = 0;
		}
		/* intialize drop in quadrant II */
		else if((rain->raindrops_so_far%4) == 1) {
			rain->rain_x[drop] = -((5*rand())%290 + 1);
			rain->rain_y[drop] = (27*rand())%290 + 1;
			rain->length[drop] = 0;
		}
		/* intialize drop in quadrant III */
		else if((rain->raindrops_so_far%4) == 2) {
			rain->rain_x[drop] = -((19*rand())%290 + 1);
			rain->rain_y[drop] = -((17*rand())%290 + 1);
			rain->length[drop] = 0;
		}
		/* intialize drop in quadrant IV */
		else {
			rain->rain_x[drop] = (13*rand())%290 + 1;
			rain->rain_y[drop] = -((23*rand())%290 + 1);
			rain->length[drop] = 0;
		}
		rain->raindrops_so_far++;
	}

	/* lengthen drop to five pixels (one each time), then start new drop */
	for(drop = 0; drop < rain->raindrops_so_far; drop++) {
		rain->length[drop] += 3;
		if(rain->length[drop] >= 15) {

			/* start drop length over after length of 6, new coords and length */
			rain->length[drop] = 0;	
					/* intialize drop in quadrant I */
			if((drop%4) == 0) {
				rain->rain_x[drop] = (23*rand())%290 + 1;
				rain->rain_y[drop] = (17*rand())%290 + 1;
				rain->length[drop] = 0;
			}
			/* intialize drop in quadrant II */
			else if((drop%4) == 1) {
				rain->rain_x[drop] = -((19*rand())%290 + 1);
				rain->rain_y[drop] = (7*rand())%290 + 1;
				rain->length[drop] = 0;
			}
			/* intialize drop in quadrant III */
			else if((drop%4) == 2) {
				rain->rain_x[drop] = -((5*rand())%290 + 1);
				rain->rain_y[drop] = -((3*rand())%290 + 1);
				rain->length[drop] = 0;
			}
			/* intialize drop in quadrant IV */
			else {
				rain->rain_x[drop] = (19 * rand())%290 + 1;
				rain->rain_y[drop] = -((13*rand())%290 + 1);
				rain->length[drop] = 0;
			}
		}
	}

	for(drop = 0; drop < rain->raindrops_so_far; drop++) {
		GP142_lineXY(NAVY_BLUE,
									rain->rain_x[drop], rain->rain_y[drop] - rain->length[drop],
									rain->rain_x[drop], 
											rain->rain_y[drop] - rain->length[drop] - 5,
									RAIN_WIDTH);

	}


}

/*****************************************************************************

     INSTRUCTIONS

*****************************************************************************/

void draw_instructions(submenu_data *current, rain_data *rain) {

	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text					*/
	int text_size   = 14;								/* size of text font							 	  */

	GP142_printfXY (WHITE, text_offset, -GP142_YMAX+30, text_size,
		"Click a button to control the program.");

	if(!(rain->raining))
	GP142_printfXY (NAVY_BLUE, -GP142_XMAX/2, GP142_YMAX-40, 2 * text_size,
		"Welcome to SR-520... (kind of)");

	else {
		GP142_printfXY (NAVY_BLUE, -GP142_XMAX/2, GP142_YMAX-40, 2 * text_size,
			"Welcome to SR-520...");
		GP142_printfXY (NAVY_BLUE, -GP142_XMAX/2 + 65, 
																								 GP142_YMAX-70, 2 * text_size,
			"that's more like it!");
	}

	if(current->instruction_type == CAR_CANT_ENTER){
		GP142_printfXY (BLACK, -15, -50, text_size,
			"Please wait until the road is safe and the car enters");
		current->ignore_click = TRUE;
		current->tick_counter++;
	}

	/* error when instructions displayed for too long */
	if(current->tick_counter > TRIAL_TIME)
		current->error_message = TRUE;

	/* print error, but allow to turn off with a new choice */
	if(current->tick_counter > TRIAL_TIME && current->error_message == TRUE){
		GP142_printfXY (BLACK, -10, -60, text_size,
			"It's not gonna happen, please choose another option...");
		current->ignore_click = FALSE;			/* start accepting clicks */
		current->instruction_type = FALSE;	/* take of instructions   */
		current->number_of_cars = current->car_counter;		/*reset number of cars*/
	}
}

/*****************************************************************************

     BUTTONS AND MOUSE CLICKS

*****************************************************************************/


/* Draws the buttons */
void draw_buttons(submenu_data *current)
{	
	int i;													/* iteration counter							 */
	int text_offset = MENU_LEFT+10;	/* offset of button text in button */
	int font_size = 12;							/* size of button text						 */

	
	/* draw rectangles for each button */
	for(i = 0; i < NUMBER_BUTTONS; i++) {
		GP142_rectangleXY(NAVY_BLUE,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */

	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(NAVY_BLUE, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(NAVY_BLUE, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(NAVY_BLUE, text_offset, i, font_size, "New Color");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(NAVY_BLUE, text_offset, i, font_size, "How Many?");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(NAVY_BLUE, text_offset, i, font_size, "Rain!!");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(NAVY_BLUE, text_offset, i, font_size, "UW Crew");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(NAVY_BLUE, text_offset, i, font_size, "Quit");

	/* if submenu status is TRUE, draw submenu */
	if(current->submenu_status){
		/* only update submenu when not scrolling in */
		if(!current->scrolling_status) 
			draw_submenu(current);
	}
	
}

void draw_submenu(submenu_data *current)
{
	int submenu_font = 10;					/* size of submenu button text */
	int j, k;												/* iteration counters					 */
	int text_offset = SUBMENU_LEFT + 6;
	int submenu_top;								/* y coord of submenu top			 */

	/* number of butttons is amount of cars plus "all" and "back" button */
	current->number_of_buttons = current->number_of_cars_for_menu + 2;

	/* set submenu top depending on selection from the main menu */
	submenu_top = MENU_TOP - (BUTTON_HEIGHT / 2) -
		(current->menu_selection) * BUTTON_HEIGHT;

	/* scroll out submenu */
	scrolling_submenu(SCROLL_OUT, current);		

	/* print submenu when scroll (rectangle width) is large enough */
	if(current->scroll >= SUBMENU_WIDTH - SCROLL_SPEED){

		/* print a rectangle for each button */
		for(j = 0; j < current->number_of_buttons; j++)
			if(current->submenu_status == TRUE) /*Skip until TRUE(tidys up scroll)*/
				GP142_rectangleXY(NAVY_BLUE,
												SUBMENU_LEFT,
												submenu_top - j * SUBMENU_HEIGHT,
												SUBMENU_LEFT + SUBMENU_WIDTH,
												submenu_top - (j + 1)*SUBMENU_HEIGHT, 1);


		/* last three buttons need to be different, these ifs check those cases */
		if(current->menu_selection == 5) { /* change buttons for case 5 */
			for(j = 0; j < current->number_of_buttons; j++){
			 k = submenu_top - SUBMENU_HEIGHT/2 - submenu_font - j * SUBMENU_HEIGHT;
			 if(j == 0)
				 GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "More");
			 if(j == 1)
				 GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "Less");
			 if(j == current->number_of_buttons - 1)
						GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "Back");
			}
			current->submenu_status = TRUE;	/* return for recursive print */
			return; /* jump out to skip normal submenu print */
		}

		else if(current->menu_selection == 6) { /* change buttons for case 6 */
			for(j = 0; j < current->number_of_buttons; j++){
			 k = submenu_top - SUBMENU_HEIGHT/2 - submenu_font - j * SUBMENU_HEIGHT;
			 if(j == 0)
				 GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "Rain");
			 if(j == 1)
				 GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "Stop");
			 if(j == current->number_of_buttons - 1)
						GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "Back");
			}
			current->submenu_status = TRUE;	/* return for recursive print */
			return; /* jump out to skip normal submenu print */
		}

		else if(current->menu_selection == 7) { /* change buttons for case 7 */
			for(j = 0; j < current->number_of_buttons; j++){
			 k = submenu_top - SUBMENU_HEIGHT/2 - submenu_font+1 - j * SUBMENU_HEIGHT;
			 if(j == 0)
				 GP142_printfXY(NAVY_BLUE, text_offset-3, k+2, 
																									submenu_font-1, "Practice");
			 if(j == 1)
				 GP142_printfXY(NAVY_BLUE, text_offset-5, k+2, 
																									submenu_font-1, "Go Home");
			 if(j == current->number_of_buttons - 1)
						GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "Back");
			}
			current->submenu_status = TRUE;	/* return for recursive print */
			return; /* jump out to skip normal submenu print */
		}

		/* print car number for car buttons, then print "all" and "back" buttons*/
		for(j = 0; j < current->number_of_buttons; j++){
			k = submenu_top - SUBMENU_HEIGHT/2 - submenu_font - j * SUBMENU_HEIGHT;
			if(j < current->number_of_cars_for_menu)
				GP142_printfXY(NAVY_BLUE, text_offset, k, 
												submenu_font, "Car %d", j + 1);
			if(j == current->number_of_buttons - 2)
					GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "All");
			if(j == current->number_of_buttons - 1)
						GP142_printfXY(NAVY_BLUE, text_offset, k, submenu_font, "Back");
		}
	current->submenu_status = TRUE;	/* return for recursive print */
	}
}

/* Scroll out and in submenu, depending on direction */
int scrolling_submenu(int direction, submenu_data *current) {

	int j;								/* iteration counter					*/
	int submenu_top;			/* y coord of submenu top			*/

	/* set submenu top depending on selection from the main menu */
	submenu_top = MENU_TOP - (BUTTON_HEIGHT / 2) -
		(current->menu_selection) * BUTTON_HEIGHT;

	/* draw rectangles for each button, start at width 0, change the width */
	/* by SCROLL_SPEED for each tick of periodic event in main, gets bigger*/
	/* or smaller depending on the direction in the function call					 */
	if(current->scroll <= SUBMENU_WIDTH){
			for(j = 0; j < current->number_of_buttons; j++){
				GP142_rectangleXY(NAVY_BLUE,
												SUBMENU_LEFT,
												submenu_top - j * SUBMENU_HEIGHT,
												SUBMENU_LEFT + current->scroll,
												submenu_top - (j + 1)*SUBMENU_HEIGHT, 1);
				
			}
		if(direction == SCROLL_OUT){
			current->scroll+=SCROLL_SPEED;
			current->submenu_status = SCROLLIN_OUT; /* calls submenu to draw when */
			return FALSE;														/*  rectangle is large enough */
		}																					/*  in draw submenu	function  */				
		if(direction == SCROLL_IN){
			current->scroll-=SCROLL_SPEED;
			if(current->scroll <= 0){
				current->submenu_status = SCROLLIN_IN; /* turns off submenu display */
				return FALSE;	/* continues scroll process */
			}
		}
	}
	return TRUE;		/* return this value to halt scrolling process */
}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse[2], submenu_data *current)
{
	int submenu_buttons;
	int submenu_top;

	
	if(current->scrolling_status)		/* ignore clicks during srcoll in */
		return OTHER_CLICK;

	if(current->ignore_click)		/* ignore clicks during other activities */
		return OTHER_CLICK;

	submenu_buttons = current->number_of_buttons;

	/* set submenu top depending on selection from the main menu */
	submenu_top = MENU_TOP - (BUTTON_HEIGHT / 2) -
		(current->menu_selection) * BUTTON_HEIGHT;

	/* return mainmenu button click, remove any error messages */
	if ((mouse[0] > MENU_LEFT) &&
			(mouse[0] < MENU_LEFT + BUTTON_WIDTH) &&
			(mouse[1] < MENU_TOP) &&
			(mouse[1] > MENU_TOP - (NUMBER_BUTTONS*BUTTON_HEIGHT))){
		current->error_message = FALSE;
		current->tick_counter = 0;    /* take off error message, reset counter */
		return BUTTON_CLICK;
	}

	/* return submenu button click, remove any error messages */
	if ((mouse[0] > SUBMENU_LEFT) && 
			(mouse[0] < SUBMENU_LEFT + SUBMENU_WIDTH) && 
			(mouse[1] < submenu_top) &&
			(mouse[1] > submenu_top - (submenu_buttons * SUBMENU_HEIGHT)) &&
			current->submenu_status == TRUE){
		current->error_message = FALSE;
		current->tick_counter = 0;    /* take off error message, reset counter */
		return SUBMENU_CLICK;
	}

	/* return extranious click */
	else
		return OTHER_CLICK;
}



/* Process button click at location (x, y) on main menu */
int handle_button(int mouse[2], car_data *cars, submenu_data *current)
{
	int buttonNum;		/* number of selected button */

	/* Complain if x coordinate couldn't be inside a button */
	if (mouse[0] < MENU_LEFT || mouse[0] > MENU_LEFT + BUTTON_WIDTH){
		something_is_wrong();
		return FALSE;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - mouse[1])/BUTTON_HEIGHT;

	switch(buttonNum) {
	case 0:
		/* go case, give number of cars to submenu draw function, draw submenu  */
		current->menu_selection = 0;
		current->number_of_cars_for_menu = current->number_of_cars;
		draw_submenu(current);
		break;
	case 1:
		/* stop case, give number of cars to submenu draw function, draw submenu*/
		current->menu_selection = 1;
		current->number_of_cars_for_menu = current->number_of_cars;
		draw_submenu(current);
		break;
		
	case 2:
		/* speed up case, give number of cars to submenu function, draw submenu */
		current->menu_selection = 2;
		current->number_of_cars_for_menu = current->number_of_cars;
		draw_submenu(current);
		break;
	case 3:
		/* slow down, give number of cars to submenu function, draw submenu		  */
		current->menu_selection = 3;
		current->number_of_cars_for_menu = current->number_of_cars;
		draw_submenu(current);
		break;		
	case 4:
		/* change color, give number of cars to submenu function, draw submenu  */
		current->menu_selection = 4;
		current->number_of_cars_for_menu = current->number_of_cars;
		draw_submenu(current);
		break;		
	case 5:
		/* change number of cars, give max number of cars to submenu function,  */
		/* draw submenu with buttons to select up to max number of cars avail.  */
		current->menu_selection = 5;
		current->number_of_cars_for_menu = 1; /* creates a submenu of 3 buttons */
		draw_submenu(current);														  
		break;
	case 6:
		/* make it rain */
		current->menu_selection = 6;
		current->number_of_cars_for_menu = 1; /* creates a submenu of 3 buttons */
		draw_submenu(current);														  
		break;
	case 7:
		/* crew boat */
		current->menu_selection = 7;
		current->number_of_cars_for_menu = 1; /* creates a submenu of 3 buttons */
		draw_submenu(current);														  
		break;
	case 8:
		/* force event loop to quit */
		return TRUE;	
		break;
	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
return FALSE;
}

/* Process button click at location (x, y) on submenu, update car attribs */
int handle_submenu(int mouse[2], car_data *cars, 
											submenu_data *current, rain_data *rain)
{
	int i;						/* iteration counter																*/
	int submenu_top;	/* y value for top of submenu												*/
	int update_value;	/* holds a number to determine update of car attrib */

	void initialize_random_numbers();

	/* Complain if x coordinate couldn't be inside a submenu button */
	if (mouse[0] < SUBMENU_LEFT || mouse[0] > SUBMENU_LEFT + SUBMENU_WIDTH) {
		something_is_wrong();
		return FALSE;
	}

	/* set submenu top depending on selection from the main menu */
	submenu_top = MENU_TOP - (BUTTON_HEIGHT / 2) -
									((current->menu_selection) * BUTTON_HEIGHT);
	
	/* Convert y coordinate to button number from top. */
 	current->submenu_selection = (submenu_top - mouse[1]) / SUBMENU_HEIGHT;
	
	/* Assign update_value with appropriate value for switch process */
	if(current->submenu_selection < current->number_of_cars_for_menu)
		 update_value = current->submenu_selection;
	if(current->submenu_selection >= current->number_of_cars_for_menu){
		if(current->submenu_selection == current->number_of_buttons - 2)
			update_value = ALL;
		if(current->submenu_selection == current->number_of_buttons - 1)
			update_value = BACK;
	}

	/* update car attributes for given main menu selection */
	switch (current->menu_selection) {
	case 0:
		/* go */

		/* if individual car selected, give speed stored from when the stoppped */
		if((update_value < current->number_of_cars_for_menu)
			&& (update_value >= 0)){
			cars->car_speed[update_value] = cars->speed_holder[update_value];
			if(cars->car_speed[update_value] <= 0) /* if still stopped, give speed*/
				cars->car_speed[update_value] = rand()%MAXIMUM_SPEED + 1;
		}

		/* do the same as above, but for all cars */
		if(update_value == ALL)
			for (i=0; i<current->number_of_cars_for_menu; i++){
			 cars->car_speed[i] = cars->speed_holder[i];
			 if(cars->car_speed[i] <= 0) /*if still stopped, give speed*/
				 cars->car_speed[i] = rand()%MAXIMUM_SPEED + 1;
			}

		/* if back selected, cause scroll in */
		if(update_value == BACK){
			/* prepare for scroll in by setting rectangle width to button width */
			current->scroll = SUBMENU_WIDTH;
			return TRUE;
		}
		break;

	case 1:
		/* stop */

		/* for indivdual car selected, store current speed, then set speed to 0 */
		if((update_value < current->number_of_cars_for_menu) 
				&& (update_value >= 0)) {
			cars->speed_holder[update_value] = cars->car_speed[update_value];
			cars->car_speed[update_value] = 0;
		}

		/* do the same as above, but for all cars */ 
		if(update_value == ALL)
			for (i=0; i<current->number_of_cars_for_menu; i++) {
				cars->speed_holder[i] = cars->car_speed[i];
				cars->car_speed[i] = 0;
			}

		/* if back selected, cause scroll in */
		if(update_value == BACK){
			/* prepare for scroll in by setting rectangle width to button width */
			current->scroll = SUBMENU_WIDTH;
			return TRUE;
		}
		break;
		
	case 2:
		/* speed up */

		/* for individual car selected, add one to speed */
		if((update_value < current->number_of_cars_for_menu) 
					&& (update_value >= 0)){

			cars->car_speed[update_value] += 1; 

			/* test within bounds of speed */
			if(cars->car_speed[update_value] <= 0) 
				cars->car_speed[update_value] = 0;
			if(cars->car_speed[update_value] >= MAXIMUM_SPEED) 
				cars->car_speed[update_value] = MAXIMUM_SPEED;
		}

		/* do the same as above, but for all cars */
		if(update_value == ALL){
			for (i=0; i<current->number_of_cars_for_menu; i++){
				cars->car_speed[i] = cars->car_speed[i] + 1;

				/* test within bounds of speed */
				if(cars->car_speed[i] <= 0) 
					cars->car_speed[i] = 0;
				if(cars->car_speed[i] >= MAXIMUM_SPEED) 
					cars->car_speed[i] = MAXIMUM_SPEED;
			}
		}

		/* if back selected, cause scroll in */
		if(update_value == BACK){
			/* prepare for scroll in by setting rectangle width to button width */
			current->scroll = SUBMENU_WIDTH;
			return TRUE;
		}
		break;

	case 3:
		/* slow down */

		/* for individual car selected, subtract one from speed */
		if((update_value < current->number_of_cars_for_menu) 
					&& (update_value >= 0)){
			
			cars->car_speed[update_value] -= 1;

			/* test within bounds of speed */
			if(cars->car_speed[update_value] <= 0) 
				cars->car_speed[update_value] = 0;
			if(cars->car_speed[update_value] >= MAXIMUM_SPEED) 
				cars->car_speed[update_value] = MAXIMUM_SPEED;
		}

		/* do the same as above, but for all cars */
		if(update_value == ALL){
			for (i=0; i<current->number_of_cars_for_menu; i++){
				cars->car_speed[i] = cars->car_speed[i] - 1;

				/* test within bounds of speed */
				if(cars->car_speed[i] <= 1) 
					cars->car_speed[i] = 1;
				if(cars->car_speed[i] >= MAXIMUM_SPEED) 
					cars->car_speed[i] = MAXIMUM_SPEED;
			}
		}

		/* if back selected, cause scroll in */
		if(update_value == BACK){
			/* prepare for scroll in by setting rectangle width to button width */
			current->scroll = SUBMENU_WIDTH;
			return TRUE;
		}
		break;
		
	case 4:
		/* change car color */

		/* for car selected, step through colors one at a time */
		if((update_value < current->number_of_cars_for_menu) 
					&& (update_value >= 0))
			cars->car_color[update_value] = 
															(cars->car_color[update_value] + 1)%MAX_COLORS;

		/* do the same as above, but for all cars */
		if(update_value == ALL){
			for (i=0; i<current->number_of_cars_for_menu; i++)
				cars->car_color[i] = rand()%MAX_COLORS;
		}

		/* if back selected, cause scroll in */
		if(update_value == BACK){
			/* prepare for scroll in by setting rectangle width to button width */
			current->scroll = SUBMENU_WIDTH;
			return TRUE;
		}
		break;
		
	case 5:
		/* change number of cars */

		/* increase cars by one, check within bounds */
		if(update_value == 0){
			current->number_of_cars += 1;
			if(current->number_of_cars >= MAX_VEHICLES)
				current->number_of_cars = MAX_VEHICLES;
		}																							

		/* (not really ALL) decrease by cars one, check within bounds */
		if(update_value == ALL){
			current->number_of_cars -= 1;
			if(current->number_of_cars <= 1)
				current->number_of_cars = 1;

			current->car_counter -= 1;
			if(current->car_counter <= 0)
				current->car_counter = 0;

		}

		/* if back selected, cause scroll in */
		if(update_value == BACK){
			/* prepare for scroll in by setting rectangle width to button width */
			current->scroll = SUBMENU_WIDTH;
			return TRUE;
		}
		break;

	case 6:
		/* change rain status */

		/* make it rain */
		if(update_value == 0)
			rain->raining = TRUE;																					

		/* cheat a little, (not really ALL) stop rain */
		if(update_value == ALL) {
			rain->raining = FALSE;
			rain->increment = 0;
		}

		/* if back selected, cause scroll in */
		if(update_value == BACK){
			/* prepare for scroll in by setting rectangle width to button width */
			current->scroll = SUBMENU_WIDTH;
			return TRUE;
		}
		break;

	case 7:
		/* crew boat */

		/* add crew boat */
		if(update_value == 0)
			current->boat_status = TRUE;																					

		/* cheat a little, (not really ALL) take away crew boat */
		if(update_value == ALL) {
			current->boat_status = FALSE;
			current->boat_center_x = GP142_XMAX + 35;		/* reset x coord	*/
			current->boat_center_y = -GP142_YMAX + 20;	/* reset y coord	*/
		}

		/* if back selected, cause scroll in */
		if(update_value == BACK){
			/* prepare for scroll in by setting rectangle width to button width */
			current->scroll = SUBMENU_WIDTH;
			return TRUE;
		}
		break;

	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
	
return FALSE; /*shouldn't happen, but if so, will cause the submenu to go in*/
}


/*****************************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

*****************************************************************************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y, int *sky_color) {

	/* Move the sun to the left.  If it reaches the edge, move it	to the	*/
	/*  other side and down a bit.  Change sky colors as it drops.				*/
	(*sun_x) -= 7;		
	if (*sun_x < -GP142_XMAX - 30) {
		*sun_x = - *sun_x ;
		(*sun_y) -= 15;
		*sky_color = ICE_BLUE;
		if (*sun_y < GP142_YMAX/2) {
			*sky_color = MAGENTA;
			(*sun_x) += 4; /* slow down sun when it is behind mountains */
		}
		if (*sun_y < 0) {
			*sky_color = BLACK;
			(*sun_y) += 15; /* stop sun from setting when it is already dark... */
		}
	}
}


/* Update all vehicle x, y coordinates */
void move_traffic(car_data *cars, submenu_data *current) {
	int trial_x;	/* trial value for position */
	int i;				/* iteration counter				*/

	/* update the coordinates of each car relative to below conditions */
	for(i = 0; i < current->number_of_cars; i++){
			
		/* check proximity to others cars, slow down if too fast */
		proximity(i, 75, cars, current);

		/* assign trial x coord according to updated speed */
		trial_x = cars->car_x[i] + (cars->car_direction[i] * cars->car_speed[i]);

		/* if it moves off the edge, appears on other side */
		if (trial_x > GP142_XMAX + CAR_WIDTH)
				trial_x = trial_x - 2*GP142_XMAX - CAR_WIDTH;
		if (trial_x < -GP142_XMAX - CAR_WIDTH)
				trial_x = trial_x + 2*GP142_XMAX + CAR_WIDTH;

		cars->car_x[i] = trial_x;
		trial_x = 0;
	}
}

/* compute the distance to another car, return value relavent to highway */
int dist_to_next_car(int car1, int car2, 
										   car_data *cars, submenu_data *current) {

	if(cars->highway[car1] == 1 && cars->highway[car2] == 1)
		return ((cars->car_x[car1] - cars->car_x[car2]));
	if(cars->highway[car1] == 2 && cars->highway[car2] == 2)
		return (-(cars->car_x[car1] - cars->car_x[car2]));
	else return NON_APPLIC;
}

/* if cars too close, adjust speed of car ahead, return false if too close */
int proximity(int car_checked_from, int how_close, 
								car_data *cars, submenu_data *current) {

	int j; /* iteration counter */

	/* store distance from car checked to all other cars */
	for(j = 0; j < current->number_of_cars; j++){

		cars->distance_to_others[j] = 
			dist_to_next_car(car_checked_from, j, cars, current);
	}

	/* if cars distance to others is within how close parameter, */
	/*  adjust speed to the car within how close  							 */
	for(j = 0; j < current->number_of_cars; j++){

		if((cars->distance_to_others[j] > 0) && 
					(cars->distance_to_others[j] <= how_close))

			if(cars->highway[car_checked_from] == cars->highway[j]) {

				if(cars->car_speed[j] > 0) 
				cars->car_speed[car_checked_from] = cars->car_speed[j];

				if(cars->car_speed[j] == 0){
					cars->speed_holder[car_checked_from] = 
																			cars->car_speed[car_checked_from];
					cars->car_speed[car_checked_from] = 0; /* retain speed if stopped */
				}
				/* if within how close, back out of range */
				if(cars->distance_to_others[car_checked_from] <= how_close)
					if(cars->highway[car_checked_from] == 1)
						cars->car_x[car_checked_from] += 3;/*slow down out of "howclose"*/
					if(cars->highway[car_checked_from] == 2)
						cars->car_x[car_checked_from] -= 3; /* same for highway 2	  		*/
				return TRUE;
			}
	}
	return FALSE; /* when no cars are too close to car checked*/
}

int is_road_taken(int car_checked_for, car_data *cars, submenu_data *current){

	int j;						 /* iteration counter */
	int safe_area;		 /* one bound for area checked for entering cars */
	int extended_edge; /* other bound */

	safe_area = GP142_XMAX - SAFE_ZONE;
	extended_edge = GP142_XMAX + SAFE_ZONE; /* extend check to offscreen area */

	for(j = 0; j < current->number_of_cars; j++){
			if((cars->car_x[j] <= extended_edge) && (cars->car_x[j] >= safe_area) ||
				 (cars->car_x[j] >= -extended_edge) && (cars->car_x[j] <= -safe_area))
				return TRUE; /* when it is not safe to enter roadway */
	}
	return FALSE; /* when it is safe to enter roadway */
}

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(car_data *cars, submenu_data *current) {
	int car_arr_num, speed;	/* car array number, speed  */

	car_arr_num = current->car_counter;
	initialize_random_numbers();

	if(car_arr_num < current->number_of_cars) {
	
		/* don't intitailize new car unless road is open */
		if(is_road_taken(car_arr_num, cars, current)){
			draw_car(cars, current);
			current->instruction_type = CAR_CANT_ENTER;  /* display instructions */
			return;
		}
			current->instruction_type = FALSE; /* stop displaying instructions */
			current->ignore_click = FALSE;		 /* don't ignore clicks anymore  */
			if (rand()%2 == 0){
				cars->car_x[car_arr_num] = -GP142_XMAX - CAR_WIDTH - car_arr_num;
				cars->car_y[car_arr_num] = HIGHWAY2_CENTER - 7;
				cars->car_direction[car_arr_num] = 1;
				cars->highway[car_arr_num] = 2;
			}
			else{
				cars->car_x[car_arr_num] = GP142_XMAX + CAR_WIDTH + car_arr_num;
				cars->car_y[car_arr_num] = HIGHWAY1_CENTER + 2;
				cars->car_direction[car_arr_num] = -1;
				cars->highway[car_arr_num] = 1;
			}

			speed = (10 + rand()%(MAXIMUM_SPEED/3)); 

			/* contain speed within max speed */
			if(cars->car_speed[car_arr_num] >= MAXIMUM_SPEED - 1) 
				cars->car_speed[car_arr_num] = MAXIMUM_SPEED;
			
			cars->car_speed[car_arr_num] = (abs(speed +1));
			cars->speed_holder[car_arr_num] = abs(speed +1);			 
			cars->car_color[car_arr_num] = rand()%MAX_COLORS;
			car_arr_num++;
			current->car_counter = car_arr_num;
	}
	draw_car(cars, current);
}



/* Draw a single car of the given color at (x, y) */
void draw_car(car_data *cars, submenu_data *current){
	int i;							/* iteration counter								*/
	int cars_thus_far;	/* current number of cars displayed */

	cars_thus_far = current->car_counter; /* number of cars initialized */

	/* draw each car */
	for (i=0; i<cars_thus_far; i++){
		GP142_ovalXY(cars->car_color[i], 
			cars->car_x[i] - CAR_WIDTH/2, cars->car_y[i] + CAR_HEIGHT/4, 
			cars->car_x[i] + CAR_WIDTH/2, cars->car_y[i] + CAR_HEIGHT*3/4, 
			FILL);
		GP142_rectangleXY(cars->car_color[i], 
			cars->car_x[i] - CAR_WIDTH/2, cars->car_y[i] + CAR_HEIGHT/4, 
			cars->car_x[i] + CAR_WIDTH/2, cars->car_y[i] + CAR_HEIGHT*1/2, 
			FILL);
		GP142_ovalXY(cars->car_color[i], 
			cars->car_x[i] - CAR_WIDTH/6, cars->car_y[i] + CAR_HEIGHT*3/4,
			cars->car_x[i] + CAR_WIDTH/6, cars->car_y[i] + CAR_HEIGHT, 
			FILL);
		GP142_circleXY(MED_GRAY, cars->car_x[i] - CAR_WIDTH/4, 
										cars->car_y[i] + CAR_HEIGHT/4, WHEEL_RADIUS);
		GP142_circleXY(MED_GRAY, cars->car_x[i] + CAR_WIDTH/4, 
										cars->car_y[i] + CAR_HEIGHT/4, WHEEL_RADIUS);
	}
	
}



/*****************************************************************************

     OTHER

*****************************************************************************/


/* Initialize random number generator */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}